@@ -348,6 +348,14 @@ buf_ValidateBuffers(void)
     }
 
     for (bp = cm_data.buf_freeListEndp; bp; bp=(cm_buf_t *) osi_QPrev(&bp->q)) {
+
+	if ( bp < (cm_buf_t *)cm_data.bufHeaderBaseAddress ||
+	     bp >= (cm_buf_t *)cm_data.bufDataBaseAddress) {
+	    afsi_log("cm_ValidateBuffers failure: out of range cm_buf_t pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range cm_buf_t pointers\n");
+	    return -11;
+	}
+
         if (bp->magic != CM_BUF_MAGIC) {
             afsi_log("cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC");
             fprintf(stderr, "cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC\n");
@@ -364,6 +372,14 @@ buf_ValidateBuffers(void)
     }
 
     for (bp = cm_data.buf_freeListp; bp; bp=(cm_buf_t *) osi_QNext(&bp->q)) {
+
+	if ( bp < (cm_buf_t *)cm_data.bufHeaderBaseAddress ||
+	     bp >= (cm_buf_t *)cm_data.bufDataBaseAddress) {
+	    afsi_log("cm_ValidateBuffers failure: out of range cm_buf_t pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range cm_buf_t pointers\n");
+	    return -12;
+	}
+
         if (bp->magic != CM_BUF_MAGIC) {
             afsi_log("cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC");
             fprintf(stderr, "cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC\n");
@@ -380,6 +396,14 @@ buf_ValidateBuffers(void)
     }
 
     for ( bp = cm_data.buf_redirListp; bp; bp = (cm_buf_t *) osi_QNext(&bp->q)) {
+
+	if ( bp < (cm_buf_t *)cm_data.bufHeaderBaseAddress ||
+	     bp >= (cm_buf_t *)cm_data.bufDataBaseAddress) {
+	    afsi_log("cm_ValidateBuffers failure: out of range cm_buf_t pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range cm_buf_t pointers\n");
+	    return -13;
+	}
+
         if (!(bp->qFlags & CM_BUF_QREDIR)) {
             afsi_log("CM_BUF_QREDIR not set on cm_buf_t in buf_redirListp");
             fprintf(stderr, "CM_BUF_QREDIR not set on cm_buf_t in buf_redirListp");
@@ -394,11 +418,27 @@ buf_ValidateBuffers(void)
     }
 
     for (bp = cm_data.buf_allp; bp; bp=bp->allp) {
+
+	if ( bp < (cm_buf_t *)cm_data.bufHeaderBaseAddress ||
+	     bp >= (cm_buf_t *)cm_data.bufDataBaseAddress) {
+	    afsi_log("cm_ValidateBuffers failure: out of range cm_buf_t pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range cm_buf_t pointers\n");
+	    return -14;
+	}
+
         if (bp->magic != CM_BUF_MAGIC) {
             afsi_log("cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC");
             fprintf(stderr, "cm_ValidateBuffers failure: bp->magic != CM_BUF_MAGIC\n");
             return -3;
         }
+
+	if ( bp->datap < cm_data.bufDataBaseAddress ||
+	     bp->datap >= cm_data.bufEndOfData) {
+	    afsi_log("cm_ValidateBuffers failure: out of range data pointers");
+	    fprintf(stderr, "cm_ValidateBuffers failure: out of range data pointers\n");
+	    return -15;
+	}
+
         counta++;
         bpa = bp;
 
@@ -537,6 +577,7 @@ long buf_Init(int newFile, cm_buf_ops_t *opsp, afs_uint64 nbuffers)
                     bp->redirLastAccess = 0;
                     bp->redirReleaseRequested = 0;
                     buf_ReleaseLocked(bp, TRUE);
+                    buf_DecrementUsedCount();
                 }
                 bp++;
             }
@@ -559,6 +600,7 @@ long buf_Init(int newFile, cm_buf_ops_t *opsp, afs_uint64 nbuffers)
                 bp->redirLastAccess = 0;
                 bp->redirReleaseRequested = 0;
                 buf_ReleaseLocked(bp, TRUE);
+                buf_DecrementUsedCount();
             }
             lock_ReleaseWrite(&buf_globalLock);
         }
@@ -816,6 +858,24 @@ afs_uint32 buf_CleanLocked(cm_scache_t *scp, cm_buf_t *bp, cm_req_t *reqp,
                      reqp) == 0)
     {
         release_scp = 1;
+
+        lock_ObtainWrite(&scp->rw);
+        code = cm_SyncOp(scp, NULL, bp->userp ? bp->userp : cm_rootUserp, reqp, 0,
+                         CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
+        if (code == 0) {
+            cm_SyncOpDone(scp, NULL, CM_SCACHESYNC_NEEDCALLBACK | CM_SCACHESYNC_GETSTATUS);
+        }
+        lock_ReleaseWrite(&scp->rw);
+    }
+
+    if (scp && (scp->flags & CM_SCACHEFLAG_DELETED)) {
+        _InterlockedAnd(&bp->flags, ~CM_BUF_DIRTY);
+        _InterlockedOr(&bp->flags, CM_BUF_ERROR);
+        bp->dirty_length = 0;
+        bp->error = code;
+        bp->dataVersion = CM_BUF_VERSION_BAD;
+        bp->dirtyCounter++;
+        buf_DecrementUsedCount();
     }
 
     while ((bp->flags & CM_BUF_DIRTY) == CM_BUF_DIRTY) {
@@ -843,9 +903,8 @@ afs_uint32 buf_CleanLocked(cm_scache_t *scp, cm_buf_t *bp, cm_req_t *reqp,
              * to determine if it is appropriate to fill a full chunk of data
              * when storing to the file server.
              */
-            code = (*cm_buf_opsp->Writep)(scp, &offset,
-                                          bp->dirty_length,
-                                          flags, bp->userp, reqp);
+            code = (*cm_buf_opsp->Writep)(scp, &offset, bp->dirty_length, flags,
+                                          bp->userp ? bp->userp : cm_rootUserp, reqp);
             osi_Log3(buf_logp, "buf_CleanLocked I/O on scp 0x%p buf 0x%p, done=%d", scp, bp, code);
         }
         lock_ObtainMutex(&bp->mx);
@@ -855,14 +914,15 @@ afs_uint32 buf_CleanLocked(cm_scache_t *scp, cm_buf_t *bp, cm_req_t *reqp,
 	 */
 	if (code == CM_ERROR_NOSUCHFILE || code == CM_ERROR_BADFD || code == CM_ERROR_NOACCESS ||
             code == CM_ERROR_QUOTA || code == CM_ERROR_SPACE || code == CM_ERROR_TOOBIG ||
-            code == CM_ERROR_READONLY || code == CM_ERROR_NOSUCHPATH || code == EIO){
+            code == CM_ERROR_READONLY || code == CM_ERROR_NOSUCHPATH || code == EIO ||
+            code == CM_ERROR_INVAL || code == CM_ERROR_INVAL_NET_RESP || code == CM_ERROR_UNKNOWN){
 	    _InterlockedAnd(&bp->flags, ~CM_BUF_DIRTY);
 	    _InterlockedOr(&bp->flags, CM_BUF_ERROR);
-            bp->dirty_offset = 0;
             bp->dirty_length = 0;
 	    bp->error = code;
 	    bp->dataVersion = CM_BUF_VERSION_BAD;
 	    bp->dirtyCounter++;
+            buf_DecrementUsedCount();
             break;
 	}
 
@@ -879,10 +939,14 @@ afs_uint32 buf_CleanLocked(cm_scache_t *scp, cm_buf_t *bp, cm_req_t *reqp,
 	if (reqp->flags & CM_REQ_NORETRY)
 	    break;
 
-        /* Ditto if the hardDeadTimeout or idleTimeout was reached */
+        /*
+         * Ditto if the hardDeadTimeout or idleTimeout was reached
+         * Or a fatal error is received.
+         */
         if (code == CM_ERROR_TIMEDOUT || code == CM_ERROR_ALLDOWN ||
             code == CM_ERROR_ALLBUSY || code == CM_ERROR_ALLOFFLINE ||
-            code == CM_ERROR_CLOCKSKEW) {
+            code == CM_ERROR_CLOCKSKEW || code == CM_ERROR_INVAL_NET_RESP ||
+            code == CM_ERROR_INVAL || code == CM_ERROR_UNKNOWN || code == EIO) {
             break;
         }
     }
@@ -1234,12 +1298,14 @@ buf_ExistsLocked(struct cm_scache *scp, osi_hyper_t *offsetp)
  * space from the buffer pool.  In that case, the buffer will be returned
  * without being hashed into the hash table.
  */
-long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *reqp, cm_buf_t **bufpp)
+long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *reqp,
+                      afs_uint32 flags, cm_buf_t **bufpp)
 {
     cm_buf_t *bp;	/* buffer we're dealing with */
     cm_buf_t *nextBp;	/* next buffer in file hash chain */
     afs_uint32 i;	/* temp */
     afs_uint64 n_bufs, n_nonzero, n_busy, n_dirty, n_own, n_redir;
+    int bufCreateLocked = !!(flags & BUF_GET_FLAG_BUFCREATE_LOCKED);
 
 #ifdef TESTING
     buf_ValidateBufQueues();
@@ -1254,12 +1320,14 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *req
         n_dirty = 0;
         n_redir = 0;
 
-        lock_ObtainRead(&scp->bufCreateLock);
+        if (!bufCreateLocked)
+            lock_ObtainRead(&scp->bufCreateLock);
         lock_ObtainWrite(&buf_globalLock);
         /* check to see if we lost the race */
         if (buf_ExistsLocked(scp, offsetp)) {
             lock_ReleaseWrite(&buf_globalLock);
-            lock_ReleaseRead(&scp->bufCreateLock);
+            if (!bufCreateLocked)
+                lock_ReleaseRead(&scp->bufCreateLock);
             return CM_BUF_EXISTS;
         }
 
@@ -1267,7 +1335,8 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *req
 	if (!cm_data.buf_freeListEndp)
 	{
 	    lock_ReleaseWrite(&buf_globalLock);
-            lock_ReleaseRead(&scp->bufCreateLock);
+            if (!bufCreateLocked)
+                lock_ReleaseRead(&scp->bufCreateLock);
 
             if ( RDR_Initialized )
                 goto rdr_release;
@@ -1351,7 +1420,8 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *req
                  */
                 buf_HoldLocked(bp);
                 lock_ReleaseWrite(&buf_globalLock);
-                lock_ReleaseRead(&scp->bufCreateLock);
+                if (!bufCreateLocked)
+                    lock_ReleaseRead(&scp->bufCreateLock);
 
                 /*
                  * grab required lock and clean.
@@ -1371,7 +1441,8 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *req
 
                 /* but first obtain the locks we gave up
                  * before the buf_CleanAsync() call */
-                lock_ObtainRead(&scp->bufCreateLock);
+                if (!bufCreateLocked)
+                    lock_ObtainRead(&scp->bufCreateLock);
                 lock_ObtainWrite(&buf_globalLock);
 
                 /*
@@ -1380,7 +1451,8 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *req
                  */
                 if (buf_ExistsLocked(scp, offsetp)) {
                     lock_ReleaseWrite(&buf_globalLock);
-                    lock_ReleaseRead(&scp->bufCreateLock);
+                    if (!bufCreateLocked)
+                        lock_ReleaseRead(&scp->bufCreateLock);
                     return CM_BUF_EXISTS;
                 }
 
@@ -1460,8 +1532,12 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *req
 		osi_panic("buf_GetNewLocked: TryMutex failed",__FILE__,__LINE__);
 	    }
 
+            if ( cm_data.buf_usedCount < cm_data.buf_nbuffers)
+                buf_IncrementUsedCount();
+
             lock_ReleaseWrite(&buf_globalLock);
-            lock_ReleaseRead(&scp->bufCreateLock);
+            if (!bufCreateLocked)
+                lock_ReleaseRead(&scp->bufCreateLock);
 
             *bufpp = bp;
 
@@ -1471,7 +1547,8 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *req
             return 0;
         } /* for all buffers in lru queue */
         lock_ReleaseWrite(&buf_globalLock);
-        lock_ReleaseRead(&scp->bufCreateLock);
+        if (!bufCreateLocked)
+            lock_ReleaseRead(&scp->bufCreateLock);
 
 	osi_Log2(afsd_logp, "buf_GetNewLocked: Free Buffer List has %u buffers none free; redir %u", n_bufs, n_redir);
         osi_Log4(afsd_logp, "... nonzero %u; own %u; busy %u; dirty %u", n_nonzero, n_own, n_busy, n_dirty);
@@ -1500,7 +1577,7 @@ long buf_GetNewLocked(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *req
  *
  * The scp must be unlocked when passed in unlocked.
  */
-long buf_Get(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *reqp, cm_buf_t **bufpp)
+long buf_Get(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *reqp, afs_uint32 flags, cm_buf_t **bufpp)
 {
     cm_buf_t *bp;
     long code;
@@ -1534,7 +1611,7 @@ long buf_Get(struct cm_scache *scp, osi_hyper_t *offsetp, cm_req_t *reqp, cm_buf
         }
 
         /* otherwise, we have to create a page */
-        code = buf_GetNewLocked(scp, &pageOffset, reqp, &bp);
+        code = buf_GetNewLocked(scp, &pageOffset, reqp, flags, &bp);
         switch (code) {
         case 0:
             /* the requested buffer was created */
@@ -1686,6 +1763,7 @@ void buf_SetDirty(cm_buf_t *bp, cm_req_t *reqp, afs_uint32 offset, afs_uint32 le
 {
     osi_assertx(bp->magic == CM_BUF_MAGIC, "invalid cm_buf_t magic");
     osi_assertx(bp->refCount > 0, "cm_buf_t refcount 0");
+    osi_assertx(userp != NULL, "userp is NULL");
 
     if (length == 0)
         return;
@@ -1849,20 +1927,21 @@ void buf_ReserveBuffers(afs_uint64 nbuffers)
     lock_ReleaseWrite(&buf_globalLock);
 }
 
-int buf_TryReserveBuffers(afs_uint64 nbuffers)
+afs_uint64
+buf_TryReserveBuffers(afs_uint64 nbuffers)
 {
-    int code;
+    afs_uint64 reserved;
 
     lock_ObtainWrite(&buf_globalLock);
     if (cm_data.buf_reservedBufs + nbuffers > cm_data.buf_maxReservedBufs) {
-        code = 0;
+        reserved = 0;
     }
     else {
         cm_data.buf_reservedBufs += nbuffers;
-        code = 1;
+        reserved = nbuffers;
     }
     lock_ReleaseWrite(&buf_globalLock);
-    return code;
+    return reserved;
 }
 
 /* called without global lock held, releases reservation held by
@@ -1968,10 +2047,10 @@ long buf_Truncate(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp,
                 }
                 _InterlockedAnd(&bufp->flags, ~CM_BUF_DIRTY);
                 bufp->error = 0;
-                bufp->dirty_offset = 0;
                 bufp->dirty_length = 0;
                 bufp->dataVersion = CM_BUF_VERSION_BAD;	/* known bad */
                 bufp->dirtyCounter++;
+                buf_DecrementUsedCount();
             }
             else {
                 /* don't set dirty, since dirty implies
@@ -2066,11 +2145,11 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
                 _InterlockedAnd(&bp->flags, ~CM_BUF_DIRTY);
                 _InterlockedOr(&bp->flags, CM_BUF_ERROR);
                 bp->error = CM_ERROR_BADFD;
-                bp->dirty_offset = 0;
                 bp->dirty_length = 0;
                 bp->dataVersion = CM_BUF_VERSION_BAD;	/* known bad */
                 bp->dirtyCounter++;
                 lock_ReleaseMutex(&bp->mx);
+                buf_DecrementUsedCount();
             } else if (!(scp->flags & CM_SCACHEFLAG_RO)) {
                 if (code) {
                     goto skip;
@@ -2097,6 +2176,8 @@ long buf_FlushCleanPages(cm_scache_t *scp, cm_user_t *userp, cm_req_t *reqp)
                             buf_HoldLocked(nbp);
                         buf_ReleaseLocked(bp, TRUE);
                         didRelease = 1;
+                        if (bp->dataVersion != CM_BUF_VERSION_BAD)
+                            buf_DecrementUsedCount();
                         buf_Recycle(bp);
                     }
                 }
@@ -2142,6 +2223,7 @@ long buf_InvalidateBuffers(cm_scache_t * scp)
     for (bp = cm_data.buf_fileHashTablepp[i]; bp; bp = bp->fileHashp) {
         if (cm_FidCmp(&bp->fid, &scp->fid) == 0) {
             bp->dataVersion = CM_BUF_VERSION_BAD;
+            buf_DecrementUsedCount();
             found = 1;
         }
     }
@@ -2210,14 +2292,16 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
              * it back in order to determine whether or not it is in
              * fact dirty.
              */
-            lock_ObtainRead(&buf_globalLock);
             if (bp->qFlags & CM_BUF_QREDIR) {
-                osi_Log1(buf_logp,"buf_CleanVnode buffer held by redirector bp 0x%p", bp);
+                lock_ObtainRead(&buf_globalLock);
+                if (bp->qFlags & CM_BUF_QREDIR) {
+                    osi_Log1(buf_logp,"buf_CleanVnode buffer held by redirector bp 0x%p", bp);
 
-                /* Retrieve single extent from the redirector */
-                buf_RDRShakeAnExtentFree(bp, reqp);
+                    /* Retrieve single extent from the redirector */
+                    buf_RDRShakeAnExtentFree(bp, reqp);
+                }
+                lock_ReleaseRead(&buf_globalLock);
             }
-            lock_ReleaseRead(&buf_globalLock);
 
             lock_ObtainMutex(&bp->mx);
             if ((bp->flags & CM_BUF_DIRTY)) {
@@ -2230,6 +2314,7 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
 
                 switch (code) {
                 case CM_ERROR_NOSUCHFILE:
+                case CM_ERROR_INVAL:
                 case CM_ERROR_BADFD:
                 case CM_ERROR_NOACCESS:
                 case CM_ERROR_QUOTA:
@@ -2238,6 +2323,8 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
                 case CM_ERROR_READONLY:
                 case CM_ERROR_NOSUCHPATH:
                 case EIO:
+                case CM_ERROR_INVAL_NET_RESP:
+                case CM_ERROR_UNKNOWN:
                     /*
                      * Apply the previous fatal error to this buffer.
                      * Do not waste the time attempting to store to
@@ -2245,11 +2332,11 @@ long buf_CleanVnode(struct cm_scache *scp, cm_user_t *userp, cm_req_t *reqp)
                      */
                     _InterlockedAnd(&bp->flags, ~CM_BUF_DIRTY);
                     _InterlockedOr(&bp->flags, CM_BUF_ERROR);
-                    bp->dirty_offset = 0;
                     bp->dirty_length = 0;
                     bp->error = code;
                     bp->dataVersion = CM_BUF_VERSION_BAD;
                     bp->dirtyCounter++;
+                    buf_DecrementUsedCount();
                     break;
                 case CM_ERROR_TIMEDOUT:
                 case CM_ERROR_ALLDOWN:
@@ -2450,7 +2537,7 @@ long buf_RDRBuffersExist(cm_fid_t *fidp)
     afs_uint32 i;
     long found = 0;
 
-    if (!RDR_Initialized)
+    if (!RDR_Initialized || cm_directIO)
         return 0;
 
     i = BUF_FILEHASH(fidp);
@@ -2473,6 +2560,9 @@ long buf_ClearRDRFlag(cm_scache_t *scp, char *reason)
     afs_uint32 bcount = 0;
     afs_uint32 i;
 
+    if (!RDR_Initialized || cm_directIO)
+        return 0;
+
     i = BUF_FILEHASH(fidp);
 
     lock_ObtainWrite(&scp->rw);
@@ -2522,7 +2612,6 @@ long buf_CleanDirtyBuffers(cm_scache_t *scp)
 	    lock_ObtainMutex(&bp->mx);
 	    _InterlockedAnd(&bp->cmFlags, ~CM_BUF_CMSTORING);
 	    _InterlockedAnd(&bp->flags, ~CM_BUF_DIRTY);
-            bp->dirty_offset = 0;
             bp->dirty_length = 0;
 	    _InterlockedOr(&bp->flags, CM_BUF_ERROR);
 	    bp->error = VNOVNODE;
@@ -2534,6 +2623,7 @@ long buf_CleanDirtyBuffers(cm_scache_t *scp)
 	    }
 	    lock_ReleaseMutex(&bp->mx);
 	    buf_Release(bp);
+            buf_DecrementUsedCount();
 	}
     }
     return 0;
