@@ -734,6 +734,7 @@ AFSQueryDirectory( IN PIRP Irp)
         {
 
             ULONG ulBytesRemainingInBuffer;
+	    BOOLEAN bUseFileInfo = FALSE;
 
             //
             // Drop the DirOpenReferenceCount held during a prior
@@ -986,9 +987,18 @@ AFSQueryDirectory( IN PIRP Irp)
 
                                 ulAdditionalAttributes = FILE_ATTRIBUTE_DIRECTORY;
                             }
+			    else if ( AFSIgnoreReparsePointToFile())
+			    {
+
+				bUseFileInfo = TRUE;
+			    }
                         }
 
-                        ulAdditionalAttributes |= FILE_ATTRIBUTE_REPARSE_POINT;
+			if ( bUseFileInfo == FALSE)
+			{
+
+			    ulAdditionalAttributes |= FILE_ATTRIBUTE_REPARSE_POINT;
+			}
 
                         break;
                     }
@@ -1044,9 +1054,23 @@ AFSQueryDirectory( IN PIRP Irp)
                 }
                 case FileDirectoryInformation:
                 {
+
                     pDirInfo = (PFILE_DIRECTORY_INFORMATION)&pBuffer[ ulNextEntry];
 
-                    if( BooleanFlagOn( pDirEntry->Flags, AFS_DIR_ENTRY_FAKE))
+		    if( bUseFileInfo)
+		    {
+
+			pDirInfo->CreationTime = stFileInfo.CreationTime;
+			pDirInfo->LastWriteTime = stFileInfo.LastWriteTime;
+			pDirInfo->LastAccessTime = stFileInfo.LastAccessTime;
+			pDirInfo->ChangeTime = stFileInfo.ChangeTime;
+
+			pDirInfo->EndOfFile = stFileInfo.EndOfFile;
+			pDirInfo->AllocationSize = stFileInfo.AllocationSize;
+
+			pDirInfo->FileAttributes = stFileInfo.FileAttributes | ulAdditionalAttributes;
+		    }
+		    else if( BooleanFlagOn( pDirEntry->Flags, AFS_DIR_ENTRY_FAKE))
                     {
 
                         pDirInfo->CreationTime = pFcb->ObjectInformation->CreationTime;
@@ -1699,6 +1723,12 @@ AFSSnapshotDirectory( IN AFSFcb *Fcb,
                 pSnapshotEntry->NameHash = (ULONG)pDirEntry->CaseSensitiveTreeEntry.HashIndex;
 
                 pSnapshotEntry++;
+
+                if ( pDirEntry->ObjectInformation != NULL)
+                {
+
+                    KeQueryTickCount( &pDirEntry->ObjectInformation->LastAccessCount);
+                }
             }
             else
             {
@@ -2268,7 +2298,9 @@ try_exit:
 
         if( pDirEnum != NULL)
         {
-            ExFreePool( pDirEnum);
+
+	    AFSLibExFreePoolWithTag( pDirEnum,
+				     AFS_GENERIC_MEMORY_3_TAG);
         }
     }
 
