@@ -60,7 +60,6 @@ DriverEntry( PDRIVER_OBJECT DriverObject,
 
     NTSTATUS ntStatus = STATUS_SUCCESS;
     AFSDeviceExt    *pDeviceExt;
-    ULONG ulTimeIncrement = 0;
     UNICODE_STRING uniDeviceName;
     ULONG ulIndex = 0;
     UNICODE_STRING uniRoutine;
@@ -147,7 +146,14 @@ DriverEntry( PDRIVER_OBJECT DriverObject,
 
             AFSRtlSetSaclSecurityDescriptor = (PAFSRtlSetSaclSecurityDescriptor)MmGetSystemRoutineAddress( &uniRoutine);
         }
+
 #endif
+
+        RtlInitUnicodeString( &uniRoutine,
+                              L"RtlSetGroupSecurityDescriptor");
+
+        AFSRtlSetGroupSecurityDescriptor = (PAFSRtlSetGroupSecurityDescriptor)MmGetSystemRoutineAddress( &uniRoutine);
+
         ntStatus = AFSCreateDefaultSecurityDescriptor();
 
         if( !NT_SUCCESS( ntStatus))
@@ -159,7 +165,7 @@ DriverEntry( PDRIVER_OBJECT DriverObject,
         }
 
         //
-        // Initilize the control device
+        // Initialize the control device
         //
 
         RtlInitUnicodeString( &uniDeviceName,
@@ -168,7 +174,7 @@ DriverEntry( PDRIVER_OBJECT DriverObject,
         ntStatus = IoCreateDevice( DriverObject,
                                    sizeof( AFSDeviceExt),
                                    &uniDeviceName,
-                                   FILE_DEVICE_DISK_FILE_SYSTEM,
+                                   FILE_DEVICE_NETWORK_FILE_SYSTEM,
                                    0,
                                    FALSE,
                                    &AFSLibraryDeviceObject);
@@ -200,18 +206,12 @@ DriverEntry( PDRIVER_OBJECT DriverObject,
         }
 
         //
-        // Initialize the worker thread pool
+        // Initialize the worker thread pool counts.
         //
 
-        ntStatus = AFSInitializeWorkerPool();
+        pDeviceExt->Specific.Library.WorkerCount = 0;
 
-        if( !NT_SUCCESS( ntStatus))
-        {
-
-            AFSPrint("AFS DriverEntry Failed to initialize worker pool Status %08lX\n", ntStatus);
-
-            try_return( ntStatus);
-        }
+        pDeviceExt->Specific.Library.IOWorkerCount = 0;
 
         //
         // Fill in the dispatch table
@@ -258,16 +258,11 @@ try_exit:
 
             AFSPrint("AFSLibrary DriverEntry failed to initialize %08lX\n", ntStatus);
 
-            if( AFSLibraryDeviceObject != NULL)
-            {
-
-                AFSRemoveWorkerPool();
-            }
-
             if( AFSRegistryPath.Buffer != NULL)
             {
 
-                ExFreePool( AFSRegistryPath.Buffer);
+		AFSLibExFreePoolWithTag( AFSRegistryPath.Buffer,
+					 AFS_GENERIC_MEMORY_13_TAG);
             }
 
             if( AFSLibraryDeviceObject != NULL)
@@ -279,10 +274,12 @@ try_exit:
             }
         }
     }
-    __except( AFSExceptionFilter( GetExceptionCode(), GetExceptionInformation()) )
+    __except( AFSExceptionFilter( __FUNCTION__, GetExceptionCode(), GetExceptionInformation()) )
     {
 
         AFSPrint( "EXCEPTION - AFS DriverEntry\n");
+
+        AFSDumpTraceFilesFnc();
     }
 
     return ntStatus;
@@ -292,6 +289,7 @@ void
 AFSUnload( IN PDRIVER_OBJECT DriverObject)
 {
 
+    UNREFERENCED_PARAMETER(DriverObject);
     if( AFSGlobalRoot != NULL)
     {
 
@@ -312,14 +310,17 @@ AFSUnload( IN PDRIVER_OBJECT DriverObject)
     if( AFSRegistryPath.Buffer != NULL)
     {
 
-        ExFreePool( AFSRegistryPath.Buffer);
+	AFSLibExFreePoolWithTag( AFSRegistryPath.Buffer,
+				 AFS_GENERIC_MEMORY_13_TAG);
     }
 
     AFSCloseLibrary();
 
     if( AFSDefaultSD != NULL)
     {
-        ExFreePool( AFSDefaultSD);
+
+	AFSLibExFreePoolWithTag( AFSDefaultSD,
+				 AFS_GENERIC_MEMORY_27_TAG);
     }
 
     if( AFSLibraryDeviceObject != NULL)
