@@ -849,13 +849,18 @@ afs_klog(khm_handle identity,
 
             memset(&increds, 0, sizeof(increds));
 
-            pkrb5_cc_get_principal(context, k5cc, &client_principal);
-            i = krb5_princ_realm(context, client_principal)->length;
-            if (i > MAXKTCREALMLEN-1)
-                i = MAXKTCREALMLEN-1;
-            StringCchCopyNA(realm_of_user, ARRAYLENGTH(realm_of_user),
-                            krb5_princ_realm(context, client_principal)->data,
-                            i);
+            r = pkrb5_cc_get_principal(context, k5cc, &client_principal);
+            if (!r) {
+                i = krb5_princ_realm(context, client_principal)->length;
+                if (i > MAXKTCREALMLEN-1)
+                    i = MAXKTCREALMLEN-1;
+                StringCchCopyNA(realm_of_user, ARRAYLENGTH(realm_of_user),
+                                krb5_princ_realm(context, client_principal)->data,
+                                i);
+            } else {
+                _reportf(L"krb5_cc_get_principal returns code %d", r);
+                goto try_krb4;
+            }
         } else {
             _reportf(L"khm_krb5_initialize returns code %d", r);
             goto try_krb4;
@@ -864,7 +869,8 @@ afs_klog(khm_handle identity,
         increds.client = client_principal;
         increds.times.endtime = 0;
         /* Ask for DES since that is what V4 understands */
-        increds.keyblock.enctype = ENCTYPE_DES_CBC_CRC;
+	if (method == AFS_TOKEN_KRB524)
+	    increds.keyblock.enctype = ENCTYPE_DES_CBC_CRC;
 
 #ifdef KRB5_TC_NOTICKET
         flags = KRB5_TC_OPENCLOSE;
@@ -1046,9 +1052,11 @@ afs_klog(khm_handle identity,
         atoken.kvno = RXKAD_TKT_TYPE_KERBEROS_V5;
         atoken.startTime = k5creds->times.starttime;
         atoken.endTime = k5creds->times.endtime;
-        memcpy(&atoken.sessionKey,
-               k5creds->keyblock.contents,
-               k5creds->keyblock.length);
+	if (tkt_DeriveDesKey(k5creds->keyblock.enctype,
+			     k5creds->keyblock.contents,
+			     k5creds->keyblock.length,
+			     &atoken.sessionKey))
+	    goto cleanup;
         atoken.ticketLen = k5creds->ticket.length;
         memcpy(atoken.ticket, k5creds->ticket.data, atoken.ticketLen);
 
@@ -1425,7 +1433,7 @@ afs_get_cellconfig(char *cell, afs_conf_cell *cellconfig, char *local_cell)
                                 (void*) cellconfig);
 
     if (linkedCell[0])
-        cellconfig->linkedCell = _strdup(linkedCell);
+        cellconfig->linkedCell = strdup(linkedCell);
 
     return rc;
 }
