@@ -31,7 +31,7 @@ osi_UFSOpen(afs_dcache_id_t *ainode)
     AFS_STATCNT(osi_UFSOpen);
     if (cacheDiskType != AFS_FCACHE_TYPE_UFS)
 	osi_Panic("UFSOpen called for non-UFS cache\n");
-    afile = osi_AllocSmallSpace(sizeof(struct osi_file));
+    afile = (struct osi_file *)osi_AllocSmallSpace(sizeof(struct osi_file));
     AFS_GUNLOCK();
     code = VFS_VGET(cacheDev.mp, (ino_t) ainode->ufs, &vp);
     AFS_GLOCK();
@@ -157,7 +157,7 @@ afs_osi_Read(struct osi_file *afile, int offset, void *aptr, afs_int32 asize)
 	if (!afs_shuttingdown)
 	    osi_Panic("osi_Read called with null param");
 	else
-	    return -EIO;
+	    return EIO;
     }
 
     if (offset != -1)
@@ -175,9 +175,7 @@ afs_osi_Read(struct osi_file *afile, int offset, void *aptr, afs_int32 asize)
     } else {
 	afs_Trace2(afs_iclSetp, CM_TRACE_READFAILED, ICL_TYPE_INT32, resid,
 		   ICL_TYPE_INT32, code);
-	if (code > 0) {
-	    code = -code;
-	}
+	code = -1;
     }
     return code;
 }
@@ -207,11 +205,8 @@ afs_osi_Write(struct osi_file *afile, afs_int32 offset, void *aptr,
 	afile->offset += code;
 	if (afile->offset > afile->size)
 	    afile->size = afile->offset;
-    } else {
-	if (code > 0) {
-	    code = -code;
-	}
-    }
+    } else
+	code = -1;
 
     if (afile->proc)
 	(*afile->proc) (afile, code);
