@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -9,38 +9,29 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-RCSID
-    ("$Header: /cvs/openafs/src/bucoord/vol_sets.c,v 1.11.6.4 2007/10/31 04:09:24 shadow Exp $");
-
 #include <afs/stds.h>
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#endif
-#include <errno.h>
+
+#include <roken.h>
+
 #include <afs/budb_client.h>
 #include <afs/cmd.h>
 #include <afs/com_err.h>
 #include <afs/bubasics.h>
+
 #include "bc.h"
 #include "error_macros.h"
+#include "bucoord_internal.h"
+#include "bucoord_prototypes.h"
 
 /* code to manage volumesets
  * specific to the ubik database implementation
  */
 
-afs_int32 bc_UpdateVolumeSet();
 extern struct bc_config *bc_globalConfig;
 extern struct udbHandleS udbHandle;
 extern char *whoami;
-extern struct bc_volumeSet *bc_FindVolumeSet(struct bc_config *cf, char *name);
-extern void FreeVolumeSet(struct bc_volumeSet *avset);
 
+static int ListVolSet(struct bc_volumeSet *aset);
 
 /* ------------------------------------
  * command level routines
@@ -60,7 +51,7 @@ extern void FreeVolumeSet(struct bc_volumeSet *avset);
 int
 bc_AddVolEntryCmd(struct cmd_syndesc *as, void *arock)
 {
-    register int code;
+    int code;
     char *volSetName, *serverName, *partitionName, *volRegExp;
     udbClientTextP ctPtr;
     struct bc_volumeSet *tset;
@@ -132,8 +123,8 @@ int
 bc_AddVolSetCmd(struct cmd_syndesc *as, void *arock)
 {
     /* parm 0 is vol set name */
-    register int code;
-    register struct cmd_item *ti;
+    int code;
+    struct cmd_item *ti;
     udbClientTextP ctPtr;
     afs_int32 flags;
 
@@ -190,7 +181,7 @@ bc_AddVolSetCmd(struct cmd_syndesc *as, void *arock)
 int
 bc_DeleteVolEntryCmd(struct cmd_syndesc *as, void *arock)
 {
-    register int code;
+    int code;
     afs_int32 entry;
     char *vsname;
     udbClientTextP ctPtr;
@@ -278,8 +269,8 @@ int
 bc_DeleteVolSetCmd(struct cmd_syndesc *as, void *arock)
 {
     /* parm 0 is vol set name */
-    register int code;
-    register struct cmd_item *ti;
+    int code;
+    struct cmd_item *ti;
     udbClientTextP ctPtr;
     afs_int32 c;
     afs_int32 flags, tosave = 0;
@@ -361,8 +352,8 @@ int
 bc_ListVolSetCmd(struct cmd_syndesc *as, void *arock)
 {
     /* parm 0 is optional volume set to display */
-    register struct bc_volumeSet *tset;
-    register struct cmd_item *ti;
+    struct bc_volumeSet *tset;
+    struct cmd_item *ti;
     int code = 0;
 
     code = bc_UpdateVolumeSet();
@@ -372,7 +363,7 @@ bc_ListVolSetCmd(struct cmd_syndesc *as, void *arock)
     }
 
     /* figure out volume set to list */
-    if (ti = as->parms[0].items) {
+    if ((ti = as->parms[0].items)) {
 	/* for each volume set in the command item list */
 	for (; ti; ti = ti->next) {
 	    tset = bc_FindVolumeSet(bc_globalConfig, ti->data);
@@ -402,9 +393,9 @@ bc_ListVolSetCmd(struct cmd_syndesc *as, void *arock)
  * ------------------------------------
  */
 
-bc_ClearVolumeSets()
+int
+bc_ClearVolumeSets(void)
 {
-    struct udbHandleS *uhptr = &udbHandle;
     struct bc_volumeSet *vsPtr, *vsNextPtr, **vsPrev;
 
     extern struct bc_config *bc_globalConfig;
@@ -433,30 +424,27 @@ bc_ClearVolumeSets()
  */
 
 int
-bc_ParseVolumeSet()
+bc_ParseVolumeSet(void)
 {
-    static char rn[] = "bc_ParseVolumeSet";	/*Routine name */
     char tbuffer[1024];		/*Buffer for reading config file */
     char vsname[256];		/*Volume set name */
     char serverName[256];	/*Server name */
     char partName[256];		/*Partition name */
-    register struct bc_volumeEntry *tve;	/*Ptr to generated volume spec struct */
-    register struct bc_volumeSet *tvs;	/*Ptr to volume set struct */
+    struct bc_volumeEntry *tve;	/*Ptr to generated volume spec struct */
+    struct bc_volumeSet *tvs = NULL;	/*Ptr to volume set struct */
     struct bc_volumeEntry **ppve, *pve;
     struct bc_volumeSet **ppvs, *pvs;
-    register afs_int32 code;	/*Generalized return code */
+    afs_int32 code;	/*Generalized return code */
     char *tp;			/*Result of fgets(), malloc() */
     int readHeader;		/*Is next thing to read a volume set hdr? */
 
     udbClientTextP ctPtr;
-    register FILE *stream;
-    struct bc_config *configPtr;
+    FILE *stream;
 
     extern struct bc_config *bc_globalConfig;
 
     ctPtr = &bc_globalConfig->configText[TB_VOLUMESET];
     stream = ctPtr->textStream;
-    configPtr = bc_globalConfig;
 
     /*
      * Open up the volume set configuration file, fail if it can't be done.
@@ -508,7 +496,7 @@ bc_ParseVolumeSet()
 	    tvs->next = (struct bc_volumeSet *)0;
 	} /*r */
 	else {			/*e */
-	    /* Scan a volume name entry, which contains the server name, 
+	    /* Scan a volume name entry, which contains the server name,
 	     * partition pattern, and volume pattern.
 	     */
 	    code = sscanf(tbuffer, "%s %s %s", serverName, partName, vsname);
@@ -520,8 +508,8 @@ bc_ParseVolumeSet()
 		continue;
 	    }
 
-	    /* The line just read in is a volume spec.  Create a new volume 
-	     * spec record, then get the rest of the information regarding 
+	    /* The line just read in is a volume spec.  Create a new volume
+	     * spec record, then get the rest of the information regarding
 	     * the host, and stuff everything into place.
 	     */
 	    tve = (struct bc_volumeEntry *)
@@ -578,7 +566,7 @@ bc_ParseVolumeSet()
 	}
     }				/*forever loop */
 
-    /* If we hit an EOF in the middle of a volume set record, we bitch and 
+    /* If we hit an EOF in the middle of a volume set record, we bitch and
      * moan.
      */
     if (!readHeader)
@@ -595,21 +583,20 @@ bc_ParseVolumeSet()
  *	save the current volume set information to disk
  */
 
-bc_SaveVolumeSet()
+int
+bc_SaveVolumeSet(void)
 {
-    register afs_int32 code = 0;
-    register struct bc_volumeSet *tset;
-    register struct bc_volumeEntry *tentry;
+    afs_int32 code = 0;
+    struct bc_volumeSet *tset;
+    struct bc_volumeEntry *tentry;
 
     udbClientTextP ctPtr;
-    register FILE *stream;
-    struct bc_config *configPtr;
+    FILE *stream;
 
     extern struct bc_config *bc_globalConfig;
 
     ctPtr = &bc_globalConfig->configText[TB_VOLUMESET];
     stream = ctPtr->textStream;
-    configPtr = bc_globalConfig;
 
     /* must be locked */
     if (ctPtr->lockHandle == 0)
@@ -656,7 +643,7 @@ bc_SaveVolumeSet()
 }
 
 afs_int32
-bc_UpdateVolumeSet()
+bc_UpdateVolumeSet(void)
 {
     struct udbHandleS *uhptr = &udbHandle;
     udbClientTextP ctPtr;
