@@ -1,7 +1,7 @@
 /*
  * Copyright 2006, Sine Nomine Associates and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -15,28 +15,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/tviced/serialize_state.c,v 1.1.4.1 2006/08/02 19:01:23 shadow Exp $");
-
-#include <stdio.h>
-#include <stdlib.h>		/* for malloc() */
-#include <time.h>		/* ANSI standard location for time stuff */
-#ifdef AFS_NT40_ENV
-#include <fcntl.h>
-#include <io.h>
-#else
-#include <sys/time.h>
-#include <sys/file.h>
-#endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#endif
-#endif
-#include <afs/assert.h>
-#include <sys/stat.h>
+#include <roken.h>
 
 #include <afs/stds.h>
 
@@ -53,11 +32,9 @@ RCSID
 #include <afs/acl.h>
 #include <afs/ptclient.h>
 #include <afs/prs_fs.h>
-#include <afs/auth.h>
 #include <afs/afsutil.h>
 #include <rx/rx.h>
 #include <afs/cellconfig.h>
-#include <stdlib.h>
 
 #include "../viced/viced_prototypes.h"
 #include "../viced/viced.h"
@@ -65,36 +42,6 @@ RCSID
 #include "../viced/callback.h"
 #include "serialize_state.h"
 
-/*@+fcnmacros +macrofcndecl@*/
-#ifdef O_LARGEFILE
-#ifdef S_SPLINT_S
-extern off64_t afs_lseek(int FD, off64_t O, int F);
-#endif /*S_SPLINT_S */
-#define afs_lseek(FD, O, F)	lseek64(FD, (off64_t)(O), F)
-#define afs_stat		stat64
-#define afs_fstat		fstat64
-#define afs_open		open64
-#define afs_fopen		fopen64
-#define afs_ftruncate           ftruncate64
-#define afs_mmap                mmap64
-#ifdef AFS_AIX_ENV
-extern void * mmap64();  /* ugly hack since aix build env appears to be somewhat broken */
-#endif
-#else /* !O_LARGEFILE */
-#ifdef S_SPLINT_S
-extern off_t afs_lseek(int FD, off_t O, int F);
-#endif /*S_SPLINT_S */
-#define afs_lseek(FD, O, F)	lseek(FD, (off_t)(O), F)
-#define afs_stat		stat
-#define afs_fstat		fstat
-#define afs_open		open
-#define afs_fopen		fopen
-#define afs_ftruncate           ftruncate
-#define afs_mmap                mmap
-#endif /* !O_LARGEFILE */
-/*@=fcnmacros =macrofcndecl@*/
-
-
 #ifdef AFS_DEMAND_ATTACH_FS
 
 /*
@@ -108,13 +55,16 @@ extern off_t afs_lseek(int FD, off_t O, int F);
  * have been written so that it will be very simple to go
  * back to standard I/O for just those poorly written platforms
  */
-#define FS_STATE_USE_MMAP
-
+#ifndef AFS_NT40_ENV
+#define FS_STATE_USE_MMAP 1
+#endif
 
 #ifdef FS_STATE_USE_MMAP
 #define FS_STATE_INIT_FILESIZE (8 * 1024 * 1024)  /* truncate to 8MB initially */
+#ifndef AFS_NT40_ENV
 #include <sys/mman.h>
 #endif
+#endif
 
 static int fs_stateCreateDump(struct fs_dump_state * state);
 static int fs_stateLoadDump(struct fs_dump_state * state);
@@ -144,9 +94,20 @@ static int fs_stateFree(struct fs_dump_state * state);
 extern afsUUID FS_HostUUID;
 extern char cml_version_number[];
 
+int
+fs_stateFileOpen(struct fs_dump_state *state)
+{
+#ifdef AFS_NT40_ENV
+    return(state->fd != -1);
+#else
+    return(state->fd >= 0);
+#endif
+}
+
+
 /*
  * demand attach fs
- * save all fileserver state 
+ * save all fileserver state
  */
 int
 fs_stateSave(void)
@@ -155,7 +116,7 @@ fs_stateSave(void)
     struct fs_dump_state state;
 
     /* save and restore need to be atomic wrt other host package operations */
-    H_LOCK; 
+    H_LOCK;
 
     ViceLog(0, ("fs_stateSave: commencing fileserver state dump\n"));
 
@@ -177,13 +138,13 @@ fs_stateSave(void)
      * BUT, this still has one flaw -- what do we do about rx worker threads that
      * are blocked in the host package making an RPC call to a cm???
      *
-     * perhaps we need a refcounter that keeps track of threads blocked in rpc calls
-     * with H_LOCK dropped (and the host struct likely left in an inconsistent state)
-     *
-     * or better yet, we need to associate a state machine with each host object
-     * (kind of like demand attach Volume structures).
-     *
-     * sigh. I suspect we'll need to revisit this issue
+     * currently we try to detect if a host struct is in an inconsistent state
+     * when we go to save it to disk, and just skip the hosts that we think may
+     * be inconsistent (see h_isBusy_r in host.c). This has the problem of causing
+     * more InitCallBackState's when we come back up, but the number of hosts in
+     * such a state should be small. In the future, we could try to lock hosts
+     * (with some deadline so we don't wait forever) before serializing, but at
+     * least for now it does not seem worth the trouble.
      */
 
     if (fs_state.options.fs_state_verify_before_save) {
@@ -231,7 +192,7 @@ fs_stateSave(void)
 
     if (fs_stateCommitDump(&state)) {
 	ViceLog(0, ("fs_stateSave: error: dump commit failed\n"));
-	ret = 1; 
+	ret = 1;
 	goto done;
     }
 
@@ -243,8 +204,8 @@ fs_stateSave(void)
 		    state.fn));
     }
 
- done:
-    if (state.fd >= 0)
+  done:
+    if (fs_stateFileOpen(&state))
 	fs_stateCloseDump(&state);
     fs_stateFree(&state);
     H_UNLOCK;
@@ -353,14 +314,14 @@ fs_stateCreateDump(struct fs_dump_state * state)
     char savedump[MAXPATHLEN];
     struct afs_stat status;
 
-    afs_snprintf(savedump, sizeof(savedump), "%s.old", state->fn);
+    snprintf(savedump, sizeof(savedump), "%s.old", state->fn);
 
     if (afs_stat(state->fn, &status) == 0) {
-	renamefile(state->fn, savedump);
+	rk_rename(state->fn, savedump);
     }
 
-    if (((fd = afs_open(state->fn, 
-			O_RDWR | O_CREAT | O_TRUNC, 
+    if (((fd = afs_open(state->fn,
+			O_RDWR | O_CREAT | O_TRUNC,
 			S_IRUSR | S_IWUSR)) == -1) ||
 	(afs_fstat(fd, &status) == -1)) {
 	ViceLog(0, ("fs_stateCreateDump: failed to create state dump file '%s'\n",
@@ -446,7 +407,7 @@ fs_stateCommitDump(struct fs_dump_state * state)
     }
 #endif
 
-    /* ensure that all pending data I/Os for the state file have been committed 
+    /* ensure that all pending data I/Os for the state file have been committed
      * _before_ we make the metadata I/Os */
     if (fs_stateSync(state)) {
 	ViceLog(0, ("fs_stateCommitDump: failed to sync changes to disk\n"));
@@ -458,8 +419,8 @@ fs_stateCommitDump(struct fs_dump_state * state)
     /* XXX madvise may not exist on all platforms, so
      * we may need to add some ifdefs at some point... */
     {
-	madvise((((char *)state->mmap.map) + sizeof(struct fs_state_header)), 
-		state->mmap.size - sizeof(struct fs_state_header), 
+	madvise((((char *)state->mmap.map) + sizeof(struct fs_state_header)),
+		state->mmap.size - sizeof(struct fs_state_header),
 		MADV_DONTNEED);
     }
 #endif
@@ -565,7 +526,7 @@ fs_stateWrite(struct fs_dump_state * state,
 	    goto done;
 	}
     }
-	    
+
     memcpy(state->mmap.cursor, buf, len);
     fs_stateIncCursor(state, len);
 #else
@@ -634,11 +595,21 @@ fs_stateWriteV(struct fs_dump_state * state,
 	fs_stateIncCursor(state, iov[i].iov_len);
     }
 #else
+#ifndef AFS_NT40_ENV
     if (writev(state->fd, iov, niov) != len) {
 	ViceLog(0, ("fs_stateWriteV: write failed\n"));
 	ret = 1;
 	goto done;
     }
+#else /* AFS_NT40_ENV */
+    for (i=0; i < niov; i++) {
+        if (write(state->fd, iov[i].iov_base, iov[i].iov_len) != iov[i].iov_len) {
+            ViceLog(0, ("fs_stateWriteV: write failed\n"));
+            ret = 1;
+            goto done;
+        }
+    }
+#endif /* AFS_NT40_ENV */
 #endif
 
  done:
@@ -669,11 +640,21 @@ fs_stateReadV(struct fs_dump_state * state,
 	fs_stateIncCursor(state, iov[i].iov_len);
     }
 #else
+#ifndef AFS_NT40_ENV
     if (readv(state->fd, iov, niov) != len) {
 	ViceLog(0, ("fs_stateReadV: read failed\n"));
 	ret = 1;
 	goto done;
     }
+#else
+    for (i=0; i < niov; i++) {
+        if (read(state->fd, iov[i].iov_base, iov[i].iov_len) != iov[i].iov_len) {
+            ViceLog(0, ("fs_stateReadV: read failed\n"));
+            ret = 1;
+            goto done;
+        }
+    }
+#endif /* AFS_NT40_ENV */
 #endif
 
  done:
@@ -745,9 +726,7 @@ fs_stateResizeFile(struct fs_dump_state * state, size_t min_add)
     int ret = 0;
     afs_foff_t inc;
 
-#ifdef FS_STATE_USE_MMAP
     fs_stateUnmapFile(state);
-#endif
 
     inc = ((min_add / FS_STATE_INIT_FILESIZE)+1) * FS_STATE_INIT_FILESIZE;
     state->file_len += inc;
@@ -758,13 +737,11 @@ fs_stateResizeFile(struct fs_dump_state * state, size_t min_add)
 	goto done;
     }
 
-#ifdef FS_STATE_USE_MMAP
     if (fs_stateMapFile(state)) {
 	ViceLog(0, ("fs_stateResizeFile: remapping memory mapped file failed\n"));
 	ret = 1;
 	goto done;
     }
-#endif
 
  done:
     return ret;
@@ -775,23 +752,13 @@ fs_stateTruncateFile(struct fs_dump_state * state)
 {
     int ret = 0;
 
-#ifdef AFS_LARGEFILE_ENV
     if (afs_ftruncate(state->fd, state->eof_offset) != 0) {
 	ret = 1;
     }
-#else
-    afs_uint32 hi, lo;
-    SplitInt64(state->eof_offset, hi, lo);
-    if (afs_ftruncate(state->fd, lo) != 0) {
-	ret = 1;
-    }
-#endif
 
     return ret;
 }
-#endif
 
-#ifdef FS_STATE_USE_MMAP
 static int
 fs_stateMapFile(struct fs_dump_state * state)
 {
@@ -809,11 +776,11 @@ fs_stateMapFile(struct fs_dump_state * state)
 	return 1;
     }
 
-    state->mmap.map = afs_mmap(NULL, 
-			       state->file_len, 
-			       flags, 
+    state->mmap.map = afs_mmap(NULL,
+			       state->file_len,
+			       flags,
 			       MAP_SHARED,
-			       state->fd, 
+			       state->fd,
 			       0);
 
     if (state->mmap.map == MAP_FAILED) {
@@ -860,9 +827,7 @@ fs_stateUnmapFile(struct fs_dump_state * state)
  done:
     return ret;
 }
-#endif /* FS_STATE_USE_MMAP */
 
-#ifdef FS_STATE_USE_MMAP
 int
 fs_stateSync(struct fs_dump_state * state)
 {
@@ -870,7 +835,6 @@ fs_stateSync(struct fs_dump_state * state)
 
     msync(state->mmap.map, state->mmap.size, MS_SYNC);
 
- done:
     return ret;
 }
 #else /* !FS_STATE_USE_MMAP */
@@ -882,7 +846,6 @@ fs_stateSync(struct fs_dump_state * state)
     if (fsync(state->fd) == -1)
 	ret = 1;
 
- done:
     return ret;
 }
 #endif /* !FS_STATE_USE_MMAP */
@@ -929,27 +892,14 @@ fs_stateSeek(struct fs_dump_state * state, afs_uint64 * offset)
 {
     int ret = 0;
     char * p;
-    afs_uint32 hi, lo;
-
-    SplitInt64(*offset, hi, lo);
 
     /* update cursor */
     p = (char *) state->mmap.map;
-#ifdef AFS_64BIT_ENV
     p += *offset;
-#else
-    p += lo;
-#endif
     state->mmap.cursor = (void *) p;
 
     /* update offset */
-#ifdef AFS_LARGEFILE_ENV
     state->mmap.offset = *offset;
-#else
-    if (hi)
-	ret = 1;
-    state->mmap.offset = lo;
-#endif
 
     return ret;
 }
@@ -958,21 +908,10 @@ int
 fs_stateSeek(struct fs_dump_state * state, afs_uint64 * offset)
 {
     int ret = 0;
-#ifndef AFS_LARGEFILE_ENV
-    afs_uint32 high, low;
-    
-    SplitInt64(*offset, high, low);
-    if (high) {
-	ret = 1;
-	goto done;
-    }
-    
-    if (afs_lseek(state->fd, low, SEEK_SET) == -1)
-	ret = 1;
-#else
+
     if (afs_lseek(state->fd, *offset, SEEK_SET) == -1)
 	ret = 1;
-#endif
+
     return ret;
 }
 #endif /* !FS_STATE_USE_MMAP */
@@ -1059,14 +998,14 @@ fs_stateCheckHeader(struct fs_state_header * hdr)
      * uncomment the following code.  uncommenting this code is _strongly discouraged_ because
      * we already make use of the version stamps in the various dump headers to deal with
      * data structure version incompatabilities.
-    else if (strncmp(hdr->server_version_string, cml_version_number, 
+    else if (strncmp(hdr->server_version_string, cml_version_number,
 		     sizeof(hdr->server_version_string)) != 0) {
 	ViceLog(0, ("fs_stateCheckHeader: dump from different server version\n"));
 	ret = 1;
     }
     */
 
-    else if (strncmp(hdr->server_version_string, cml_version_number, 
+    else if (strncmp(hdr->server_version_string, cml_version_number,
 		     sizeof(hdr->server_version_string)) != 0) {
 	ViceLog(0, ("fs_stateCheckHeader: dump from different server version ; attempting state reload anyway\n"));
     }
@@ -1081,7 +1020,7 @@ fs_stateAlloc(struct fs_dump_state * state)
     int ret = 0;
     memset(state, 0, sizeof(struct fs_dump_state));
     state->fd = -1;
-    state->fn = AFSDIR_SERVER_FSSTATE_FILEPATH;
+    state->fn = (char *)AFSDIR_SERVER_FSSTATE_FILEPATH;
     state->hdr = (struct fs_state_header *)malloc(sizeof(struct fs_state_header));
     state->h_hdr = (struct host_state_header *)malloc(sizeof(struct host_state_header));
     state->cb_hdr = (struct callback_state_header *)malloc(sizeof(struct callback_state_header));
