@@ -11,17 +11,13 @@
 #include <afs/param.h>
 
 #include <roken.h>
-#include <afs/opr.h>
-#ifdef AFS_PTHREAD_ENV
-# include <opr/lock.h>
-#endif
 
 #ifdef AFS_NT40_ENV
 #include <windows.h>
 #include <WINNT/afsevent.h>
 #endif
 
-#include <rx/rx_queue.h>
+#include <rx/xdr.h>
 #include <afs/afsint.h>
 #include <afs/prs_fs.h>
 #include <afs/nfs.h>
@@ -72,7 +68,6 @@ int udpBufSize = 0;		/* UDP buffer size for receive */
 
 int rxBind = 0;
 int rxkadDisableDotCheck = 0;
-int DoPreserveVolumeStats = 0;
 
 #define ADDRSPERSITE 16         /* Same global is in rx/rx_user.c */
 afs_uint32 SHostAddrs[ADDRSPERSITE];
@@ -231,25 +226,6 @@ vol_rxstat_userok(struct rx_call *call)
     return afsconf_SuperUser(tdir, call, NULL);
 }
 
-/**
- * Return true if this name is a member of the local realm.
- */
-static int
-vol_IsLocalRealmMatch(void *rock, char *name, char *inst, char *cell)
-{
-    struct afsconf_dir *dir = (struct afsconf_dir *)rock;
-    afs_int32 islocal = 0;	/* default to no */
-    int code;
-
-    code = afsconf_IsLocalRealmMatch(dir, &islocal, name, inst, cell);
-    if (code) {
-	ViceLog(0,
-		("Failed local realm check; code=%d, name=%s, inst=%s, cell=%s\n",
-		 code, name, inst, cell));
-    }
-    return islocal;
-}
-
 #include "AFS_component_version_number.c"
 int
 main(int argc, char **argv)
@@ -365,8 +341,6 @@ main(int argc, char **argv)
 	    rx_enablePeerRPCStats();
 	} else if (strcmp(argv[code], "-enable_process_stats") == 0) {
 	    rx_enableProcessRPCStats();
-	} else if (strcmp(argv[code], "-preserve-vol-stats") == 0) {
-	    DoPreserveVolumeStats = 1;
 	}
 #ifndef AFS_NT40_ENV
 	else if (strcmp(argv[code], "-syslog") == 0) {
@@ -488,10 +462,10 @@ main(int argc, char **argv)
 #ifdef AFS_PTHREAD_ENV
 	pthread_t tid;
 	pthread_attr_t tattr;
-	opr_Verify(pthread_attr_init(&tattr) == 0);
-	opr_Verify(pthread_attr_setdetachstate(&tattr,
-					       PTHREAD_CREATE_DETACHED) == 0);
-	opr_Verify(pthread_create(&tid, &tattr, BKGLoop, NULL) == 0);
+	osi_Assert(pthread_attr_init(&tattr) == 0);
+	osi_Assert(pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED) == 0);
+
+	osi_Assert(pthread_create(&tid, &tattr, BKGLoop, NULL) == 0);
 #else
 	PROCESS pid;
 	LWP_CreateProcess(BKGLoop, 16*1024, 3, 0, "vol bkg daemon", &pid);
@@ -507,10 +481,6 @@ main(int argc, char **argv)
 	      AFSDIR_SERVER_ETC_DIRPATH);
 	VS_EXIT(1);
     }
-
-    /* initialize audit user check */
-    osi_audit_set_user_check(tdir, vol_IsLocalRealmMatch);
-
     afsconf_GetKey(tdir, 999, &tkey);
     afsconf_BuildServerSecurityObjects(tdir, &securityClasses, &numClasses);
     if (securityClasses[0] == NULL)
